@@ -55,6 +55,7 @@ def init_lib(path):
     c_lib.pattern_ascii_pointer_1.argtypes = [ctypes.POINTER(ctypes.c_uint8)]
     c_lib.pattern_ascii_pointer_2.argtypes = []
     c_lib.pattern_ascii_pointer_len.argtypes = [ctypes.POINTER(ctypes.c_uint8), UseAsciiStringPattern]
+    c_lib.pattern_ascii_pointer_return_slice.argtypes = []
     c_lib.pattern_ffi_slice_1.argtypes = [Sliceu32]
     c_lib.pattern_ffi_slice_2.argtypes = [SliceVec3f32, ctypes.c_int32]
     c_lib.pattern_ffi_slice_3.argtypes = [SliceMutu8, callbacks.fn_SliceMutu8]
@@ -137,6 +138,7 @@ def init_lib(path):
     c_lib.pattern_ascii_pointer_1.restype = ctypes.c_uint32
     c_lib.pattern_ascii_pointer_2.restype = ctypes.POINTER(ctypes.c_uint8)
     c_lib.pattern_ascii_pointer_len.restype = ctypes.c_uint32
+    c_lib.pattern_ascii_pointer_return_slice.restype = SliceUseAsciiStringPattern
     c_lib.pattern_ffi_slice_1.restype = ctypes.c_uint32
     c_lib.pattern_ffi_slice_2.restype = Vec3f32
     c_lib.pattern_ffi_slice_delegate.restype = ctypes.c_uint8
@@ -334,6 +336,9 @@ def pattern_ascii_pointer_len(x: str, y: UseAsciiStringPattern) -> int:
         x = ctypes.cast(x, ctypes.POINTER(ctypes.c_uint8))
     return c_lib.pattern_ascii_pointer_len(x, y)
 
+def pattern_ascii_pointer_return_slice() -> SliceUseAsciiStringPattern:
+    return c_lib.pattern_ascii_pointer_return_slice()
+
 def pattern_ffi_slice_1(ffi_slice: Sliceu32) -> int:
     return c_lib.pattern_ffi_slice_1(ffi_slice)
 
@@ -420,6 +425,24 @@ class CallbackVars(object):
         return rval
 
 
+class _Iter(object):
+    """Helper for slice iterators."""
+    def __init__(self, target):
+        self.i = 0
+        self.target = target
+
+    def __iter__(self):
+        self.i = 0
+        return self
+
+    def __next__(self):
+        if self.i >= self.target.len:
+            raise StopIteration()
+        rval = self.target[self.i]
+        self.i += 1
+        return rval
+
+
 class EnumDocumented:
     """ Documented enum."""
     #  Variant A.
@@ -936,6 +959,26 @@ class SliceBool(ctypes.Structure):
     def __getitem__(self, i):
         return self.data[i]
 
+    def copied(self) -> SliceBool:
+        """Returns a shallow, owned copy of the underlying slice.
+
+        The returned object owns the immediate data, but not the targets of any contained
+        pointers. In other words, if your struct contains any pointers the returned object
+        may only be used as long as these pointers are valid. If the struct did not contain
+        any pointers the returned object is valid indefinitely."""
+        array = (ctypes.c_uint8 * len(self))()
+        ctypes.memmove(array, self.data, len(self) * ctypes.sizeof(ctypes.c_uint8))
+        rval = SliceBool(data=ctypes.cast(array, ctypes.POINTER(ctypes.c_uint8)), len=len(self))
+        rval.owned = array  # Store array in returned slice to prevent memory deallocation
+        return rval
+
+    def __iter__(self):
+        return _Iter(self)
+
+    def iter(self):
+        """Convenience method returning a value iterator."""
+        return iter(self)
+
 
 class Sliceu32(ctypes.Structure):
     # These fields represent the underlying C data layout
@@ -950,6 +993,26 @@ class Sliceu32(ctypes.Structure):
     def __getitem__(self, i) -> int:
         return self.data[i]
 
+    def copied(self) -> Sliceu32:
+        """Returns a shallow, owned copy of the underlying slice.
+
+        The returned object owns the immediate data, but not the targets of any contained
+        pointers. In other words, if your struct contains any pointers the returned object
+        may only be used as long as these pointers are valid. If the struct did not contain
+        any pointers the returned object is valid indefinitely."""
+        array = (ctypes.c_uint32 * len(self))()
+        ctypes.memmove(array, self.data, len(self) * ctypes.sizeof(ctypes.c_uint32))
+        rval = Sliceu32(data=ctypes.cast(array, ctypes.POINTER(ctypes.c_uint32)), len=len(self))
+        rval.owned = array  # Store array in returned slice to prevent memory deallocation
+        return rval
+
+    def __iter__(self):
+        return _Iter(self)
+
+    def iter(self):
+        """Convenience method returning a value iterator."""
+        return iter(self)
+
 
 class Sliceu8(ctypes.Structure):
     # These fields represent the underlying C data layout
@@ -964,6 +1027,26 @@ class Sliceu8(ctypes.Structure):
     def __getitem__(self, i) -> int:
         return self.data[i]
 
+    def copied(self) -> Sliceu8:
+        """Returns a shallow, owned copy of the underlying slice.
+
+        The returned object owns the immediate data, but not the targets of any contained
+        pointers. In other words, if your struct contains any pointers the returned object
+        may only be used as long as these pointers are valid. If the struct did not contain
+        any pointers the returned object is valid indefinitely."""
+        array = (ctypes.c_uint8 * len(self))()
+        ctypes.memmove(array, self.data, len(self) * ctypes.sizeof(ctypes.c_uint8))
+        rval = Sliceu8(data=ctypes.cast(array, ctypes.POINTER(ctypes.c_uint8)), len=len(self))
+        rval.owned = array  # Store array in returned slice to prevent memory deallocation
+        return rval
+
+    def __iter__(self):
+        return _Iter(self)
+
+    def iter(self):
+        """Convenience method returning a value iterator."""
+        return iter(self)
+
 
 class SliceMutu32(ctypes.Structure):
     # These fields represent the underlying C data layout
@@ -981,6 +1064,26 @@ class SliceMutu32(ctypes.Structure):
     def __setitem__(self, i, v: int):
         self.data[i] = v
 
+    def copied(self) -> SliceMutu32:
+        """Returns a shallow, owned copy of the underlying slice.
+
+        The returned object owns the immediate data, but not the targets of any contained
+        pointers. In other words, if your struct contains any pointers the returned object
+        may only be used as long as these pointers are valid. If the struct did not contain
+        any pointers the returned object is valid indefinitely."""
+        array = (ctypes.c_uint32 * len(self))()
+        ctypes.memmove(array, self.data, len(self) * ctypes.sizeof(ctypes.c_uint32))
+        rval = SliceMutu32(data=ctypes.cast(array, ctypes.POINTER(ctypes.c_uint32)), len=len(self))
+        rval.owned = array  # Store array in returned slice to prevent memory deallocation
+        return rval
+
+    def __iter__(self):
+        return _Iter(self)
+
+    def iter(self):
+        """Convenience method returning a value iterator."""
+        return iter(self)
+
 
 class SliceMutu8(ctypes.Structure):
     # These fields represent the underlying C data layout
@@ -998,36 +1101,84 @@ class SliceMutu8(ctypes.Structure):
     def __setitem__(self, i, v: int):
         self.data[i] = v
 
+    def copied(self) -> SliceMutu8:
+        """Returns a shallow, owned copy of the underlying slice.
+
+        The returned object owns the immediate data, but not the targets of any contained
+        pointers. In other words, if your struct contains any pointers the returned object
+        may only be used as long as these pointers are valid. If the struct did not contain
+        any pointers the returned object is valid indefinitely."""
+        array = (ctypes.c_uint8 * len(self))()
+        ctypes.memmove(array, self.data, len(self) * ctypes.sizeof(ctypes.c_uint8))
+        rval = SliceMutu8(data=ctypes.cast(array, ctypes.POINTER(ctypes.c_uint8)), len=len(self))
+        rval.owned = array  # Store array in returned slice to prevent memory deallocation
+        return rval
+
+    def __iter__(self):
+        return _Iter(self)
+
+    def iter(self):
+        """Convenience method returning a value iterator."""
+        return iter(self)
+
 
 class OptionInner(ctypes.Structure):
+    """May optionally hold a value."""
 
-    # These fields represent the underlying C data layout
     _fields_ = [
-        ("t", Inner),
-        ("is_some", ctypes.c_uint8),
+        ("_t", Inner),
+        ("_is_some", ctypes.c_uint8),
     ]
 
-    def __init__(self, t: Inner = None, is_some: int = None):
-        if t is not None:
-            self.t = t
-        if is_some is not None:
-            self.is_some = is_some
-
     @property
-    def t(self) -> Inner:
-        return ctypes.Structure.__get__(self, "t")
+    def value(self) -> Inner:
+        """Returns the value if it exists, or None."""
+        if self._is_some == 1:
+            return self._t
+        else:
+            return None
 
-    @t.setter
-    def t(self, value: Inner):
-        return ctypes.Structure.__set__(self, "t", value)
+    def is_some(self) -> bool:
+        """Returns true if the value exists."""
+        return self._is_some == 1
 
-    @property
-    def is_some(self) -> int:
-        return ctypes.Structure.__get__(self, "is_some")
+    def is_none(self) -> bool:
+        """Returns true if the value does not exist."""
+        return self._is_some != 0
 
-    @is_some.setter
-    def is_some(self, value: int):
-        return ctypes.Structure.__set__(self, "is_some", value)
+
+class SliceUseAsciiStringPattern(ctypes.Structure):
+    # These fields represent the underlying C data layout
+    _fields_ = [
+        ("data", ctypes.POINTER(UseAsciiStringPattern)),
+        ("len", ctypes.c_uint64),
+    ]
+
+    def __len__(self):
+        return self.len
+
+    def __getitem__(self, i) -> UseAsciiStringPattern:
+        return self.data[i]
+
+    def copied(self) -> SliceUseAsciiStringPattern:
+        """Returns a shallow, owned copy of the underlying slice.
+
+        The returned object owns the immediate data, but not the targets of any contained
+        pointers. In other words, if your struct contains any pointers the returned object
+        may only be used as long as these pointers are valid. If the struct did not contain
+        any pointers the returned object is valid indefinitely."""
+        array = (UseAsciiStringPattern * len(self))()
+        ctypes.memmove(array, self.data, len(self) * ctypes.sizeof(UseAsciiStringPattern))
+        rval = SliceUseAsciiStringPattern(data=ctypes.cast(array, ctypes.POINTER(UseAsciiStringPattern)), len=len(self))
+        rval.owned = array  # Store array in returned slice to prevent memory deallocation
+        return rval
+
+    def __iter__(self):
+        return _Iter(self)
+
+    def iter(self):
+        """Convenience method returning a value iterator."""
+        return iter(self)
 
 
 class SliceVec3f32(ctypes.Structure):
@@ -1043,6 +1194,26 @@ class SliceVec3f32(ctypes.Structure):
     def __getitem__(self, i) -> Vec3f32:
         return self.data[i]
 
+    def copied(self) -> SliceVec3f32:
+        """Returns a shallow, owned copy of the underlying slice.
+
+        The returned object owns the immediate data, but not the targets of any contained
+        pointers. In other words, if your struct contains any pointers the returned object
+        may only be used as long as these pointers are valid. If the struct did not contain
+        any pointers the returned object is valid indefinitely."""
+        array = (Vec3f32 * len(self))()
+        ctypes.memmove(array, self.data, len(self) * ctypes.sizeof(Vec3f32))
+        rval = SliceVec3f32(data=ctypes.cast(array, ctypes.POINTER(Vec3f32)), len=len(self))
+        rval.owned = array  # Store array in returned slice to prevent memory deallocation
+        return rval
+
+    def __iter__(self):
+        return _Iter(self)
+
+    def iter(self):
+        """Convenience method returning a value iterator."""
+        return iter(self)
+
 
 
 
@@ -1160,7 +1331,7 @@ class SimpleService:
         return c_lib.simple_service_return_slice_mut(self._ctx, )
 
     def return_string(self, ) -> str:
-        """"""
+        """ This function has no panic safeguards. If it panics your host app will be in an undefined state."""
         rval = c_lib.simple_service_return_string(self._ctx, )
         return ctypes.string_at(rval)
 
